@@ -1,7 +1,8 @@
 """
-VABY_MODELS_CVR: VABY forward models for CVR
+VABY_MODELS_CVR: VABY forward models for cerebrovascular reactivity (CVR)
 
-Forward model for CVR measurement using BOLD MRI and PETCo2
+This model is intended for CVR measurement using functional MRI (BOLD/ASL)
+in conjunction with CO2 stimulus
 
 Based on Matlab code written by Joana Pinto, December 2020, Oxford
 and adapted from Daniel Bulte 2018 script
@@ -26,24 +27,25 @@ class CvrPetCo2Model(Model):
     Inference forward model for CVR measurement using PETCo2
     """
 
-    OPTIONS = [
-        # Regressor, e.g. physiological data file containing PETCO2 measurements
-        ModelOption("regressors", "Regression data (e.g. PETCO2 or O2 time series)", type=str, default=None),
-        ModelOption("regressor_types", "Regressor types - comma separated one for each regressor. Supported types: co2, petco2, custom", type=str, default="co2"),
-        ModelOption("regressor_trs", "Regressor time resolutions", unit="s", type=ValueList, default=[0.01,]),
+    def options(self):
+        return [
+            # Regressor, e.g. physiological data file containing PETCO2 measurements
+            ModelOption("regressors", "Regression data (e.g. PETCO2 or O2 time series)", type=str, default=None),
+            ModelOption("regressor_types", "Regressor types - comma separated one for each regressor. Supported types: co2, petco2, custom", type=str, default="co2"),
+            ModelOption("regressor_trs", "Regressor time resolutions", unit="s", type=ValueList, default=[0.01,]),
 
-        # Protocol parameters
-        ModelOption("baseline", "Length of initial baseline block", unit="s", type=int, default=60),
-        ModelOption("data_start_time", "Start of MR data relative to start of regressor data - if not provided will be estimated", unit="s", type=float, default=None),
-        ModelOption("tr", "Time between MR volumes", unit="s", type=float, default=None),
-        ModelOption("air_pressure", "Barometric pressure", unit="mbar", type=int, default=1020),
+            # Protocol parameters
+            ModelOption("baseline", "Length of initial baseline block", unit="s", type=int, default=60),
+            ModelOption("data_start_time", "Start of MR data relative to start of regressor data - if not provided will be estimated", unit="s", type=float, default=None),
+            ModelOption("tr", "Time between MR volumes", unit="s", type=float, default=None),
+            ModelOption("air_pressure", "Barometric pressure", unit="mbar", type=int, default=1020),
 
-        # Model options
-        ModelOption("infer_sig0", "Infer signal offset", type=bool, default=False),
-        ModelOption("infer_delay", "Infer delay shift on regressors", type=bool, default=False),
-        #ModelOption("infer_drift", "Infer a linear drift on signal", type=bool, default=False),
-        #ModelOption("sigmoid_response", "Use sigmoid relationship between PETCO2 and CVR", type=bool, default=False)
-    ]
+            # Model options
+            ModelOption("infer_sig0", "Infer signal offset", type=bool, default=False),
+            ModelOption("infer_delay", "Infer delay shift on regressors", type=bool, default=False),
+            #ModelOption("infer_drift", "Infer a linear drift on signal", type=bool, default=False),
+            #ModelOption("sigmoid_response", "Use sigmoid relationship between PETCO2 and CVR", type=bool, default=False)
+        ]
 
     def __str__(self):
         return "CVR-PETCO2 model: %s" % __version__
@@ -101,16 +103,23 @@ class CvrPetCo2Model(Model):
         self.regressors = np.array(regressors)
         self.regressor_tpts = np.array(regressor_tpts)
 
+        # Interpolate regressors onto BOLD timeseries
+        regressors_interp = []
+        mr_timings = self.tpts()
+        for regressor, regressor_tpts in zip(regressors, regressor_tpts):
+            regressors_interp.append(np.interp(mr_timings, regressor_tpts, regressor))
+        self.regressors_interp = np.array(regressors_interp, dtype=np.float32)
+
         # Differences between timepoints for quick interpolation. Given a delay
         # time > 0 we can compute value = regressors[int(delay)] + frac(delay) * regressor_diff[int(delay)]
-        self.regressor_diffs = np.zeros(self.regressors.shape, dtype=np.float32)
-        self.regressor_diffs[:, :-1] = self.regressors[:, 1:] - self.regressors[:, :-1]
+        self.regressor_diffs = np.zeros(self.regressors_interp.shape, dtype=np.float32)
+        self.regressor_diffs[:, :-1] = self.regressors_interp[:, 1:] - self.regressors_interp[:, :-1]
 
         # Min/max values
         self.regressor_mins = np.min(self.regressors, axis=1)
         self.regressor_maxs = np.max(self.regressors, axis=1)
-        self.log.info("Regressor minimum values: %s", self.regressor_mins)
-        self.log.info("Regressor maximum values: %s", self.regressor_maxs)
+        self.log.info(" - Regressor minimum values: %s", self.regressor_mins)
+        self.log.info(" - Regressor maximum values: %s", self.regressor_maxs)
 
         # Estimate data start time
         if self.data_start_time is None:
@@ -121,12 +130,12 @@ class CvrPetCo2Model(Model):
         if self.infer_delay:
             self.params.append(get_parameter("delay", mean=0, prior_var=100, post_var=10, **options))
 
-    def _init_sig0(self, _param, _t, data):
+    def _init_sig0(self, _param, data):
         return np.mean(data, axis=-1), None
 
     def fit_glm(self, delay_min=-1, delay_max=1, delay_step=1, progress_cb=None):
-        self.log.info("GLM: Doing fitting on %i voxels", self.data_model.n_voxels)
-        bold_data = self.data_model.data_flat
+        self.log.info("GLM: Doing fitting on %i voxels", self.data_model.data_space.size)
+        bold_data = self.data_model.data_space.srcdata.flat
         t = self.tpts() # in seconds
 
         delays = np.arange(delay_min, delay_max+delay_step, delay_step, dtype=np.float32)
@@ -148,7 +157,7 @@ class CvrPetCo2Model(Model):
                 else:
                     x.append(delayed)
 
-            x.append(np.ones(self.data_model.n_tpts))
+            x.append(np.ones(self.data_model.data_space.srcdata.n_tpts))
             x = np.array(x).T
             for vox in range(bold_data.shape[0]):
                 y = bold_data[vox, :]
@@ -204,9 +213,9 @@ class CvrPetCo2Model(Model):
             delay = params[extra_param] - self.data_start_time
             extra_param += 1
 
-            # Apply time delay [W, (S), N] FIXME what is length of regressor
+            # Apply time delay [W, (S), N]
             t_delayed = (tpts - delay) / self.tr
-            t_delayed = tf.clip_by_value(t_delayed, 0, len(self.regressors[0])-1)
+            t_delayed = tf.clip_by_value(t_delayed, 0, len(self.regressors_interp[0])-1)
             t_base = tf.floor(t_delayed)
 
             # Integer index into the CO2 and diff arrays
@@ -219,7 +228,7 @@ class CvrPetCo2Model(Model):
             t_frac = None
 
         fit = 1
-        for idx, regressor in enumerate(self.regressors):
+        for idx, regressor in enumerate(self.regressors_interp):
             # Tile regressor arrays over all nodes so we can use tf.gather
             regressor = tf.tile(regressor[np.newaxis, ...], (tf.shape(t_base_idx)[0], 1))
 
@@ -253,11 +262,11 @@ class CvrPetCo2Model(Model):
         :return: Either a Numpy array of shape [N] or a Numpy array of shape
                  [W, N] for nodewise timepoints.
         """
-        return np.linspace(0, self.data_model.n_tpts, num=self.data_model.n_tpts, endpoint=False, dtype=np.float32) * self.tr
+        return np.linspace(0, self.data_model.data_space.srcdata.n_tpts, num=self.data_model.data_space.srcdata.n_tpts, endpoint=False, dtype=np.float32) * self.tr
 
     def estimate_data_start_time(self, regressor, regressor_tpts):
         # Mean time series
-        bold_data_average = np.mean(self.data_model.data_flat, axis=0)
+        bold_data_average = np.mean(self.data_model.data_space.srcdata.flat, axis=0)
 
         # Interpolate BOLD timeseries onto regressor
         mr_timings = self.tpts()
@@ -265,18 +274,18 @@ class CvrPetCo2Model(Model):
 
         _cc, delay_vols = self._cross_corr(bold_data_interp, regressor)
         self.data_start_time = -delay_vols * regressor_tpts[1] # to seconds assuming uniform spacing
-        self.log.info("Cross correlation estimated data start time: %f", self.data_start_time)
+        self.log.info(" - Cross correlation estimated data start time: %f", self.data_start_time)
 
         # Calculate the latest possible start time of the MR data
         # in case the cross correlation method returns something silly
         regressor_duration = len(regressor) * regressor_tpts[1]
         mr_duration = mr_timings[-1]
         max_time_begin = regressor_duration - mr_duration
-        self.log.debug("Regressor duration: %f", regressor_duration)
-        self.log.debug("MR duration: %f", mr_duration)
-        self.log.info("Absolute latest start time: %f", max_time_begin)
+        self.log.debug(" - Regressor duration: %f", regressor_duration)
+        self.log.debug(" - MR duration: %f", mr_duration)
+        self.log.info(" - Absolute latest start time: %f", max_time_begin)
         self.data_start_time = min(self.data_start_time, max_time_begin)
-        self.log.info("Final estimated data start time: %f", self.data_start_time)
+        self.log.info(" - Final estimated data start time: %f", self.data_start_time)
 
     def _cross_corr(self, y1, y2):
         """
@@ -359,8 +368,8 @@ class CvrPetCo2Model(Model):
         # Create a timecourse of the end tidal CO2 values at the TR's for use with CVR sigmoids
         # Make new time course at the TR resolution and normalise timecourse betwwen 0 and 1 to create EV
         block = int(round(self.tr*samp_rate))
-        ev_co2 = np.zeros((self.data_model.n_tpts,), dtype=np.float32)
-        for i in range(self.data_model.n_tpts):
+        ev_co2 = np.zeros((self.data_model.data_space.srcdata.n_tpts,), dtype=np.float32)
+        for i in range(self.data_model.data_space.srcdata.n_tpts):
             ev_co2[i] = co2_resamp[block * i + block-1]
 
         # Convert to mmHg
